@@ -1,30 +1,20 @@
 """Amazon AWS Connection."""
 
+from __future__ import annotations
+
+from email import message_from_bytes
+from email.mime.message import MIMEMessage
+
 from vine import promise, transform
 
 from kombu.asynchronous.aws.ext import AWSRequest, get_response
-
 from kombu.asynchronous.http import Headers, Request, get_client
 
-import io
 
-try:  # pragma: no cover
-    from email import message_from_bytes
-    from email.mime.message import MIMEMessage
+def message_from_headers(hdr):
+    bs = "\r\n".join("{}: {}".format(*h) for h in hdr)
+    return message_from_bytes(bs.encode())
 
-    # py3
-    def message_from_headers(hdr):  # noqa
-        bs = "\r\n".join("{}: {}".format(*h) for h in hdr)
-        return message_from_bytes(bs.encode())
-
-except ImportError:  # pragma: no cover
-    from mimetools import Message as MIMEMessage  # noqa
-
-    # py2
-    def message_from_headers(hdr):  # noqa
-        return io.BytesIO(b'\r\n'.join(
-            b'{}: {}'.format(*h) for h in hdr
-        ))
 
 __all__ = (
     'AsyncHTTPSConnection', 'AsyncConnection',
@@ -141,7 +131,7 @@ class AsyncHTTPSConnection:
 class AsyncConnection:
     """Async AWS Connection."""
 
-    def __init__(self, sqs_connection, http_client=None, **kwargs):  # noqa
+    def __init__(self, sqs_connection, http_client=None, **kwargs):
         self.sqs_connection = sqs_connection
         self._httpclient = http_client or get_client()
 
@@ -189,14 +179,14 @@ class AsyncAWSQueryConnection(AsyncConnection):
                  http_client_params=None, **kwargs):
         if not http_client_params:
             http_client_params = {}
-        AsyncConnection.__init__(self, sqs_connection, http_client,
-                                 **http_client_params)
+        super().__init__(sqs_connection, http_client,
+                         **http_client_params)
 
     def make_request(self, operation, params_, path, verb, callback=None):  # noqa
         params = params_.copy()
         if operation:
             params['Action'] = operation
-        signer = self.sqs_connection._request_signer  # noqa
+        signer = self.sqs_connection._request_signer
 
         # defaults for non-get
         signing_type = 'standard'
@@ -237,7 +227,7 @@ class AsyncAWSQueryConnection(AsyncConnection):
             ),
         )
 
-    def _on_list_ready(self, parent, markers, operation, response):  # noqa
+    def _on_list_ready(self, parent, markers, operation, response):
         service_model = self.sqs_connection.meta.service_model
         if response.status == self.STATUS_CODE_OK:
             _, parsed = get_response(
@@ -255,7 +245,7 @@ class AsyncAWSQueryConnection(AsyncConnection):
         else:
             raise self._for_status(response, response.read())
 
-    def _on_obj_ready(self, parent, operation, response):  # noqa
+    def _on_obj_ready(self, parent, operation, response):
         service_model = self.sqs_connection.meta.service_model
         if response.status == self.STATUS_CODE_OK:
             _, parsed = get_response(
@@ -265,7 +255,7 @@ class AsyncAWSQueryConnection(AsyncConnection):
         else:
             raise self._for_status(response, response.read())
 
-    def _on_status_ready(self, parent, operation, response):  # noqa
+    def _on_status_ready(self, parent, operation, response):
         service_model = self.sqs_connection.meta.service_model
         if response.status == self.STATUS_CODE_OK:
             httpres, _ = get_response(
