@@ -2,34 +2,37 @@
 
 Emulates the AMQ API for non-AMQ transports.
 """
-from __future__ import absolute_import, print_function, unicode_literals
+
+from __future__ import annotations
 
 import base64
 import socket
 import sys
 import warnings
-
 from array import array
 from collections import OrderedDict, defaultdict, namedtuple
 from itertools import count
 from multiprocessing.util import Finalize
-from time import sleep
+from queue import Empty
+from time import monotonic, sleep
+from typing import TYPE_CHECKING
 
 from amqp.protocol import queue_declare_ok_t
 
-from kombu.exceptions import ResourceError, ChannelError
-from kombu.five import Empty, items, monotonic
+from kombu.exceptions import ChannelError, ResourceError
 from kombu.log import get_logger
-from kombu.utils.encoding import str_to_bytes, bytes_to_str
+from kombu.transport import base
 from kombu.utils.div import emergency_dump_state
+from kombu.utils.encoding import bytes_to_str, str_to_bytes
 from kombu.utils.scheduling import FairCycle
 from kombu.utils.uuid import uuid
 
-from kombu.transport import base
-
 from .exchange import STANDARD_EXCHANGE_TYPES
 
-ARRAY_TYPE_H = 'H' if sys.version_info[0] == 3 else b'H'
+if TYPE_CHECKING:
+    from types import TracebackType
+
+ARRAY_TYPE_H = 'H'
 
 UNDELIVERABLE_FMT = """\
 Message could not be delivered: No queues bound to exchange {exchange!r} \
@@ -61,7 +64,7 @@ queue_binding_t = namedtuple('queue_binding_t', (
 ))
 
 
-class Base64(object):
+class Base64:
     """Base64 codec."""
 
     def encode(self, s):
@@ -79,7 +82,7 @@ class UndeliverableWarning(UserWarning):
     """The message could not be delivered to a queue."""
 
 
-class BrokerState(object):
+class BrokerState:
     """Broker state holds exchanges, queues and bindings."""
 
     #: Mapping of exchange name to
@@ -98,6 +101,7 @@ class BrokerState(object):
 
     #: The queue index is used to access directly (constant time)
     #: all the bindings of a certain queue.  It has the following structure::
+    #:
     #:     {
     #:         queue: {
     #:             (queue, exchange, routing_key),
@@ -149,12 +153,13 @@ class BrokerState(object):
         )
 
 
-class QoS(object):
+class QoS:
     """Quality of Service guarantees.
 
     Only supports `prefetch_count` at this point.
 
     Arguments:
+    ---------
         channel (ChannelT): Connection channel.
         prefetch_count (int): Initial prefetch count (defaults to 0).
     """
@@ -179,6 +184,8 @@ class QoS(object):
         self.channel = channel
         self.prefetch_count = prefetch_count or 0
 
+        # Standard Python dictionaries do not support setting attributes
+        # on the object, hence the use of OrderedDict
         self._delivered = OrderedDict()
         self._delivered.restored = False
         self._dirty = set()
@@ -205,7 +212,8 @@ class QoS(object):
         bulk 'get message' calls are preferred to many individual 'get message'
         calls - like SQS.
 
-        Returns:
+        Returns
+        -------
             int: greater than zero.
         """
         pcount = self.prefetch_count
@@ -267,6 +275,7 @@ class QoS(object):
         """Restore all unacknowledged messages at shutdown/gc collect.
 
         Note:
+        ----
             Can only be called once for each instance, subsequent
             calls will be ignored.
         """
@@ -300,10 +309,10 @@ class QoS(object):
         To be filled in for visibility_timeout style implementations.
 
         Note:
+        ----
             This is implementation optional, and currently only
             used by the Redis transport.
         """
-        pass
 
 
 class Message(base.Message):
@@ -315,7 +324,7 @@ class Message(base.Message):
         body = payload.get('body')
         if body:
             body = channel.decode_body(body, properties.get('body_encoding'))
-        super(Message, self).__init__(
+        super().__init__(
             body=body,
             channel=channel,
             delivery_tag=properties['delivery_tag'],
@@ -343,13 +352,14 @@ class Message(base.Message):
         }
 
 
-class AbstractChannel(object):
+class AbstractChannel:
     """Abstract channel interface.
 
     This is an abstract class defining the channel methods
     you'd usually want to implement in a virtual channel.
 
     Note:
+    ----
         Do not subclass directly, but rather inherit
         from :class:`Channel`.
     """
@@ -374,6 +384,7 @@ class AbstractChannel(object):
         """Delete `queue`.
 
         Note:
+        ----
             This just purges the queue, if you need to do more you can
             override this method.
         """
@@ -383,15 +394,16 @@ class AbstractChannel(object):
         """Create new queue.
 
         Note:
+        ----
             Your transport can override this method if it needs
             to do something whenever a new queue is declared.
         """
-        pass
 
     def _has_queue(self, queue, **kwargs):
         """Verify that queue exists.
 
-        Returns:
+        Returns
+        -------
             bool: Should return :const:`True` if the queue exists
                 or :const:`False` otherwise.
         """
@@ -410,6 +422,7 @@ class Channel(AbstractChannel, base.StdChannel):
     """Virtual channel.
 
     Arguments:
+    ---------
         connection (ConnectionT): The transport instance this
             channel is part of.
     """
@@ -463,17 +476,10 @@ class Channel(AbstractChannel, base.StdChannel):
 
         # instantiate exchange types
         self.exchange_types = {
-            typ: cls(self) for typ, cls in items(self.exchange_types)
+            typ: cls(self) for typ, cls in self.exchange_types.items()
         }
 
-        try:
-            self.channel_id = self.connection._avail_channel_ids.pop()
-        except IndexError:
-            raise ResourceError(
-                'No free channel ids, current={0}, channel_max={1}'.format(
-                    len(self.connection.channels),
-                    self.connection.channel_max), (20, 10),
-            )
+        self.channel_id = self._get_free_channel_id()
 
         topts = self.connection.client.transport_options
         for opt_name in self.from_transport_options:
@@ -491,7 +497,7 @@ class Channel(AbstractChannel, base.StdChannel):
         if passive:
             if exchange not in self.state.exchanges:
                 raise ChannelError(
-                    'NOT_FOUND - no exchange {0!r} in vhost {1!r}'.format(
+                    'NOT_FOUND - no exchange {!r} in vhost {!r}'.format(
                         exchange, self.connection.client.virtual_host or '/'),
                     (50, 10), 'Channel.exchange_declare', '404',
                 )
@@ -523,7 +529,7 @@ class Channel(AbstractChannel, base.StdChannel):
         queue = queue or 'amq.gen-%s' % uuid()
         if passive and not self._has_queue(queue, **kwargs):
             raise ChannelError(
-                'NOT_FOUND - no queue {0!r} in vhost {1!r}'.format(
+                'NOT_FOUND - no queue {!r} in vhost {!r}'.format(
                     queue, self.connection.client.virtual_host or '/'),
                 (50, 10), 'Channel.queue_declare', '404',
             )
@@ -678,6 +684,7 @@ class Channel(AbstractChannel, base.StdChannel):
         """Change QoS settings for this channel.
 
         Note:
+        ----
             Only `prefetch_count` is supported.
         """
         self.qos.prefetch_count = prefetch_count
@@ -700,9 +707,10 @@ class Channel(AbstractChannel, base.StdChannel):
     def _lookup(self, exchange, routing_key, default=None):
         """Find all queues matching `routing_key` for the given `exchange`.
 
-        Returns:
-            str: queue name -- must return the string `default`
-                if no queues matched.
+        Returns
+        -------
+            list[str]: queue names -- must return `[default]`
+                if default is set and no queues matched.
         """
         if default is None:
             default = self.deadletter_queue
@@ -731,7 +739,8 @@ class Channel(AbstractChannel, base.StdChannel):
         message = message.serializable()
         message['redelivered'] = True
         for queue in self._lookup(
-                delivery_info['exchange'], delivery_info['routing_key']):
+            delivery_info['exchange'],
+                delivery_info['routing_key']):
             self._put(queue, message)
 
     def _restore_at_beginning(self, message):
@@ -767,7 +776,8 @@ class Channel(AbstractChannel, base.StdChannel):
     def flow(self, active=True):
         """Enable/disable message flow.
 
-        Raises:
+        Raises
+        ------
             NotImplementedError: as flow
                 is not implemented by the base virtual implementation.
         """
@@ -808,7 +818,12 @@ class Channel(AbstractChannel, base.StdChannel):
     def __enter__(self):
         return self
 
-    def __exit__(self, *exc_info):
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_val: BaseException | None,
+        exc_tb: TracebackType | None
+    ) -> None:
         self.close()
 
     @property
@@ -835,6 +850,7 @@ class Channel(AbstractChannel, base.StdChannel):
         The value is limited to within a boundary of 0 to 9.
 
         Note:
+        ----
             Higher value has more priority.
         """
         try:
@@ -848,12 +864,28 @@ class Channel(AbstractChannel, base.StdChannel):
 
         return (self.max_priority - priority) if reverse else priority
 
+    def _get_free_channel_id(self):
+        # Cast to a set for fast lookups, and keep stored as an array
+        # for lower memory usage.
+        used_channel_ids = set(self.connection._used_channel_ids)
+
+        for channel_id in range(1, self.connection.channel_max + 1):
+            if channel_id not in used_channel_ids:
+                self.connection._used_channel_ids.append(channel_id)
+                return channel_id
+
+        raise ResourceError(
+            'No free channel ids, current={}, channel_max={}'.format(
+                len(self.connection.channels),
+                self.connection.channel_max), (20, 10),
+        )
+
 
 class Management(base.Management):
     """Base class for the AMQP management API."""
 
     def __init__(self, transport):
-        super(Management, self).__init__(transport)
+        super().__init__(transport)
         self.channel = transport.client.channel()
 
     def get_bindings(self):
@@ -868,6 +900,7 @@ class Transport(base.Transport):
     """Virtual transport.
 
     Arguments:
+    ---------
         client (kombu.Connection): The client this is a transport for.
     """
 
@@ -875,9 +908,6 @@ class Transport(base.Transport):
     Cycle = FairCycle
     Management = Management
 
-    #: Global :class:`BrokerState` containing declared exchanges and bindings.
-    state = BrokerState()
-
     #: :class:`~kombu.utils.scheduling.FairCycle` instance
     #: used to fairly drain events from channels (set by constructor).
     cycle = None
@@ -905,6 +935,8 @@ class Transport(base.Transport):
 
     def __init__(self, client, **kwargs):
         self.client = client
+        # :class:`BrokerState` containing declared exchanges and bindings.
+        self.state = BrokerState()
         self.channels = []
         self._avail_channels = []
         self._callbacks = {}
@@ -912,9 +944,7 @@ class Transport(base.Transport):
         polling_interval = client.transport_options.get('polling_interval')
         if polling_interval is not None:
             self.polling_interval = polling_interval
-        self._avail_channel_ids = array(
-            ARRAY_TYPE_H, range(self.channel_max, 0, -1),
-        )
+        self._used_channel_ids = array(ARRAY_TYPE_H)
 
     def create_channel(self, connection):
         try:
@@ -926,7 +956,11 @@ class Transport(base.Transport):
 
     def close_channel(self, channel):
         try:
-            self._avail_channel_ids.append(channel.channel_id)
+            try:
+                self._used_channel_ids.remove(channel.channel_id)
+            except ValueError:
+                # channel id already removed
+                pass
             try:
                 self.channels.remove(channel)
             except ValueError:
@@ -939,14 +973,14 @@ class Transport(base.Transport):
         # this channel is then used as the next requested channel.
         # (returned by ``create_channel``).
         self._avail_channels.append(self.create_channel(self))
-        return self     # for drain events
+        return self  # for drain events
 
     def close_connection(self, connection):
         self.cycle.close()
-        for l in self._avail_channels, self.channels:
-            while l:
+        for chan_list in self._avail_channels, self.channels:
+            while chan_list:
                 try:
-                    channel = l.pop()
+                    channel = chan_list.pop()
                 except LookupError:  # pragma: no cover
                     pass
                 else:
@@ -972,7 +1006,7 @@ class Transport(base.Transport):
     def _deliver(self, message, queue):
         if not queue:
             raise KeyError(
-                'Received message without destination queue: {0}'.format(
+                'Received message without destination queue: {}'.format(
                     message))
         try:
             callback = self._callbacks[queue]
@@ -993,7 +1027,7 @@ class Transport(base.Transport):
     def on_message_ready(self, channel, message, queue):
         if not queue or queue not in self._callbacks:
             raise KeyError(
-                'Message for queue {0!r} without consumers: {1}'.format(
+                'Message for queue {!r} without consumers: {}'.format(
                     queue, message))
         self._callbacks[queue](message)
 
